# This file is read by Autoconf.                       -*- Autoconf -*-
#
#   Berkeley Lab Checkpoint/Restart (BLCR) for Linux is Copyright (c)
#   2007, The Regents of the University of California, through Lawrence
#   Berkeley National Laboratory (subject to receipt of any required
#   approvals from the U.S. Dept. of Energy).  All rights reserved.
#
#   Portions may be copyrighted by others, as may be noted in specific
#   copyright notices within specific files.
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# $Id: acinclude.m4,v 1.82 2007/03/02 01:24:02 phargrov Exp $
AC_REVISION($Revision: 1.82 $)

# CR_IF(test,[ACTION-IF-FOUND],[ACTION-IF-FOUND])
# ------------------------------------------------------
# Basic functionality of AS_IF, but not as "optimized"
# Implemented here because m4sh is not yet documented.
AC_DEFUN([CR_IF],[
  if $1; then
    m4_default([$2], :)
  else
    m4_default([$3], :)
  fi
])

# CR_PROG_GCC([LIST],[ACTION-IF-FOUND],[ACTION-IF-NOT-FOUND])
# ------------------------------------------------------
# Find the first compiler in the list and verify that it is gcc.
AC_DEFUN([CR_PROG_GCC],[
  AC_PROG_CC($1)
  CR_IF([test "$GCC" = yes],[$2],[$3])
])

# CR_CHECK_GLIBC(MAJOR,MINOR,[ACTION-IF-FOUND],[ACTION-IF-NOT-FOUND])
# ------------------------------------------------------
# Check for glibc >= MAJOR.MINOR and sets the variable
# cr_cv_check_glibc_MAJOR_MINOR_or_higher to 'yes' or 'no'
AC_DEFUN([CR_CHECK_GLIBC],[
  cr_cvname="cr_cv_check_glibc_"$1"_"$2"_or_higher"
  AC_CACHE_CHECK([[for GNU libc version >= $1.$2]],[$cr_cvname],[
    AC_TRY_RUN([
	#include <features.h>
	#include <stdlib.h>
	#ifndef __GLIBC_PREREQ
	    #define __GLIBC_PREREQ(maj, min) \
		((__GLIBC__ << 16) + __GLIBC_MINOR__ >= ((maj) << 16) + (min))
	#endif
	extern int gnu_get_libc_version();
	int main ()
	{
	    exit((gnu_get_libc_version() && __GLIBC_PREREQ($1, $2)) ? 0 : 1);
	}
    ], [eval $cr_cvname=yes], [eval $cr_cvname=no])
  ])
  CR_IF([eval test \$$cr_cvname = yes],[$3],[$4])
])

# CR_KERNEL_TYPE(TYPE)
# ------------------------------------------------------
# Check for /boot/kernel.h and establish user-requested overrides
# if possible.
AC_DEFUN([CR_KERNEL_TYPE],[
  cr_kernel_type=`echo $1 | tr 'a-z' 'A-Z'`
  cr_header="/boot/kernel.h"
  AC_MSG_CHECKING([for $cr_header]);
  cr_result=ok
  if test \! -e "$cr_header"; then
    cr_result='missing'
  elif test \! -r "$cr_header"; then
    cr_result='not readable'
  fi
  AC_MSG_RESULT([$cr_result])
  if test "$cr_result" != "ok"; then
    AC_MSG_ERROR([You have requested '--with-kernel-type=$cr_kernel_type', but $cr_header is $cr_result.])
  fi
  cr_kernel_var="__BOOT_KERNEL_$cr_kernel_type"
  if test -z "`grep \"$cr_kernel_var\" $cr_header 2>/dev/null`"; then
    AC_MSG_ERROR([You have requested '--with-kernel-type=$cr_kernel_type', but $cr_header does not appear to support that type.])
  fi
  # The following sed command transforms all the #ifndef lines from kernel.h into
  # corresponding preprocessor flags (on one line) which select the desired kernel type.
  # Note use of [] for m4 quoting of a sed command containing [ and ]
  CR_KTYPE_CPPFLAGS=[`sed -n -e '/^#ifndef \('$cr_kernel_var'\)$/ {s//-D\1=1 /;H;}' \
			     -e '/^#ifndef \(__BOOT_KERNEL_[A-Z]*\)$/ {s//-D\1=0 /;H;}' \
			     -e '$ {x;s/[ \t\n]\+/ /g;s/^ //;s/ $//;p;}' \
			     $cr_header`]
])


# _CR_CHECK_VERSION_H(MAJOR,MINOR,DIR,FULL)
# ------------------------------------------------------
# Check if the indicated DIR contains version.h for the given
# kernel version.
# Sets cr_linux_dir_ok to 'yes' or 'no'
# Sets cr_linux_ver to the version found
# Boolean FULL determines if the version is x.y.z (no) or the
# full UTS_RELEASE string (yes).
AC_DEFUN([_CR_CHECK_VERSION_H],[
  cr_linux_dir_ok=no
  if test -r $3/include/linux/version.h; then
    SAVE_CPPFLAGS=$CPPFLAGS
    if test "$1.$2" = "2.6"; then
      cr_kbuild_extra="-DKBUILD_STR(s)=#s -DKBUILD_BASENAME=KBUILD_STR(configure) -DKBUILD_MODNAME=KBUILD_STR(configure)  -include $3/include/linux/autoconf.h"
    else
      cr_kbuild_extra=""
    fi
    CPPFLAGS="-I$3/include -D__KERNEL__ -DMODULE $cr_kbuild_extra $CR_KTYPE_CPPFLAGS $CPPFLAGS"
    if test x"$4" = xyes; then
      cr_version_code="fputs(UTS_RELEASE, f); fputc('\n', f);"
    else
      cr_version_code='fprintf(f, "%d.%d.%d\n",
				  (int)((LINUX_VERSION_CODE >> 16) & 0xff),
				  (int)((LINUX_VERSION_CODE >> 8) & 0xff),
				  (int)((LINUX_VERSION_CODE) & 0xff));'
    fi
    AC_TRY_RUN([
	#include <stdio.h>
	#include "$3/include/linux/version.h"
	#ifndef UTS_RELEASE
	  #include "$3/include/linux/utsrelease.h"
	#endif
	int main ()
	{
		FILE *f=fopen("conftestval", "w");
  		if (!f) return(1);

		${cr_version_code}

		return !((LINUX_VERSION_CODE >= KERNEL_VERSION($1,$2,0)) &&
			 (LINUX_VERSION_CODE <  KERNEL_VERSION($1,$2+1,0)));
	}
    ], [cr_linux_dir_ok=yes
        cr_linux_ver=`cat conftestval`
        CR_KBUILD_EXTRA="$cr_kbuild_extra"])
    CPPFLAGS=$SAVE_CPPFLAGS
  fi
])

# _CR_CHECK_LINUX_SRC(MAJOR,MINOR,DIR)
# ------------------------------------------------------
# Check if the indicated DIR contains source for the given
# kernel version.  Verification consists of:
# + Makefile with proper version
#  or
# + include/linux/version.h w/ proper version
# Sets cr_linux_dir_ok to 'yes' or 'no'
# Sets cr_linux_ver to the version found
AC_DEFUN([_CR_CHECK_LINUX_SRC],[
  cr_linux_ver=''
  if test -e $3/Makefile; then
    # If a .config does not exist, then make may complain if/when the main Makefile
    # tries to include it.  The -k and 2>/dev/null take care of that.
    eval cr_linux_ver=`make -k echo_kver --no-print-directory -C $3 -f - 2>/dev/null <<'_EOF_'
echo_kver:
	@echo '$(VERSION).$(PATCHLEVEL).$(SUBLEVEL)'

include Makefile
_EOF_`
    if test -z "${cr_linux_ver}"; then
      # If we got nothing, try pattern matching the required fields instead.
      # Note the use of [] for m4 quoting, since the pattern contains [ and ]
      cr_linux_ver1=[`sed -n -e '/^VERSION[ \t]*=[ \t]*\([0-9]\+\).*$/ {s//\1/p;q;}' "$3/Makefile"`]
      cr_linux_ver2=[`sed -n -e '/^PATCHLEVEL[ \t]*=[ \t]*\([0-9]\+\).*$/ {s//\1/p;q;}' "$3/Makefile"`]
      cr_linux_ver3=[`sed -n -e '/^SUBLEVEL[ \t]*=[ \t]*\([0-9]\+\).*$/ {s//\1/p;q;}' "$3/Makefile"`]
      cr_linux_ver="${cr_linux_ver1}.${cr_linux_ver2}.${cr_linux_ver3}"
    fi
  fi
  if test -z "${cr_linux_ver}"; then
    _CR_CHECK_VERSION_H([$1],[$2],[$3],[no])
  fi
  cr_linux_dir_ok=no
  # XXX: Note that requiring a non-empty SUBLEVEL ensures we don't accidentally
  # match a SuSE-type Makefile in a 2.6 build dir.  Such a setup is fine for
  # kbuild, but not for TRY_KERNEL_COMPILE until it learns to use kbuild.
  case "$cr_linux_ver" in $1.$2.?*) cr_linux_dir_ok=yes;; esac
])

# CR_FIND_LINUX_SRC(MAJOR,MINOR,[ACTION-IF-FOUND],[ACTION-IF-NOT-FOUND])
# ------------------------------------------------------
# Search for Linux src dir
# Sets cr_linux_dir_ok to yes or no
# Sets LINUX_SRC if found.
AC_DEFUN([CR_FIND_LINUX_SRC],[
  AC_MSG_CHECKING([[for Linux kernel $1.$2 source directory]])
  if test -n "$LINUX_SRC" ; then
    # Just validate the user's setting
    _CR_CHECK_LINUX_SRC([$1],[$2],[$LINUX_SRC])
  else
    # Search standard locations
    for cr_linux_dir in /usr/src/linux-`uname -r` \
			/lib/modules/`uname -r`/source \
			/lib/modules/`uname -r`/build \
			/usr/src/linux-$1.$2 \
			/usr/src/linux; do
      _CR_CHECK_LINUX_SRC([$1],[$2],[$cr_linux_dir])
      if test $cr_linux_dir_ok = yes; then
	LINUX_SRC="$cr_linux_dir"
	break
      fi
   done
  fi
  CR_IF([test $cr_linux_dir_ok = yes],
	[AC_MSG_RESULT([$LINUX_SRC (base version $cr_linux_ver)]); $3],
	[AC_MSG_RESULT([not found${LINUX_SRC:+ in $LINUX_SRC}]); $4])
])

# _CR_CHECK_LINUX_OBJ(MAJOR,MINOR,DIR)
# ------------------------------------------------------
# Check if the indicated DIR contains a build for the given
# kernel version.  Verification consists of:
# + include/linux/version.h w/ proper version
# + .config, IFF enable_kbuild==yes
# Sets cr_linux_dir_ok to 'yes' or 'no'
# Sets cr_linux_ver to the version found
AC_DEFUN([_CR_CHECK_LINUX_OBJ],[
  # Check for version.h
  _CR_CHECK_VERSION_H([$1],[$2],[$3],[yes])
  # Check for .config if required
  if test $cr_linux_dir_ok = no -o -r "$3/.config"; then
    : # OK, know enough not to check other conditions
  elif test "$1.$2" = "2.4" -a x$enable_kbuild = xyes; then
    # For 2.4 kernel, we only reject the dir if kbuild explicitly enabled
    cr_linux_dir_ok=no
  elif test "$1.$2" = "2.6" -a x$enable_kbuild != xno; then
    # For 2.4 kernel, we only accept the dir if kbuild explicitly disabled
    # Note that disabling kbuild w/ 2.6 is illegal - that is flaged elsewhere
    cr_linux_dir_ok=no
  fi
])

# CR_FIND_LINUX_OBJ(MAJOR,MINOR,[ACTION-IF-FOUND],[ACTION-IF-NOT-FOUND])
# ------------------------------------------------------
# Search for Linux obj (aka build) dir
# Sets cr_linux_dir_ok to yes or no
# Sets LINUX_OBJ if found.
AC_DEFUN([CR_FIND_LINUX_OBJ],[
  AC_MSG_CHECKING([[for Linux kernel $1.$2 build directory]])
  if test -n "${HAVE_LINUX_2_4}"; then
    # 2.4.x kernel: must have srcdir=objdir, so just verify LINUX_SRC
    if test -n "${LINUX_OBJ}"; then
      AC_MSG_WARN([ignoring --with-linux-obj/LINUX_OBJ setting for 2.4.x kernel])
    fi
    _CR_CHECK_LINUX_OBJ([$1],[$2],[$LINUX_SRC])
    if test $cr_linux_dir_ok = yes; then
      LINUX_OBJ="${cr_linux_dir}"
    else
      AC_MSG_RESULT([not found in $LINUX_SRC])
      AC_MSG_ERROR([source directory $LINUX_SRC is missing include/linux/version.h])
    fi
  elif test -n "${LINUX_OBJ}"; then
    # Just validate the user's setting
    _CR_CHECK_LINUX_OBJ([$1],[$2],[$LINUX_OBJ])
  else
    # Search standard locations
    for cr_linux_dir in "${LINUX_SRC}" \
			/lib/modules/`uname -r`/build \
			/usr/src/linux-`uname -r`-obj \
			/usr/src/linux-$1.$2-obj \
			/usr/src/linux-obj \
			/usr/src/linux-`uname -r` \
			/usr/src/linux-$1.$2 \
			/usr/src/linux; do
      _CR_CHECK_LINUX_OBJ([$1],[$2],[$cr_linux_dir])
      if test $cr_linux_dir_ok = yes; then
        LINUX_OBJ="${cr_linux_dir}"
        break
      fi
    done
  fi
  CR_IF([test $cr_linux_dir_ok = yes],
	[AC_MSG_RESULT([$LINUX_OBJ (version $cr_linux_ver)]); $3],
	[AC_MSG_RESULT([not found${LINUX_OBJ:+ in $LINUX_OBJ}]); $4])
])

# CR_CHECK_LINUX(MAJOR,MINOR)
# ------------------------------------------------------
# Check for source and build dirs of the specified MAJOR.MINOR
# version of Linux, and set LINUX_SRC and LINUX_OBJ accordingly.
# If either is already set, just verify it w/o searching.
# Any patch-level checking should be done elsewhere.
AC_DEFUN([CR_CHECK_LINUX],[
  AC_SUBST([LINUX_SRC])
  AC_SUBST([LINUX_OBJ])
  AC_SUBST([LINUX_VER])
  AC_SUBST([CR_KERNEL])
  AC_SUBST([CR_KERNEL_BASE])
  AC_SUBST([CR_MODULE_DIR])
  AC_SUBST([HAVE_LINUX_$1_$2])
  CR_FIND_LINUX_SRC([$1],[$2],
    [cr_linux_src_ver="$cr_linux_ver"
     CR_FIND_LINUX_OBJ([$1],[$2],
       [if test ${LINUX_OBJ} != ${LINUX_SRC}; then
	  case "$cr_linux_ver" in
            [${cr_linux_src_ver}|${cr_linux_src_ver}[^0-9]*])	;;	# OK (the outer [] is m4 quoting)
            *) AC_MSG_ERROR([Kernel source and build directories appear to have different versions (see above)])
          esac
	fi
        eval "HAVE_LINUX_$1_$2"=yes
        LINUX_VER="$cr_linux_ver"
        CR_KERNEL=`echo $cr_linux_ver | tr - _`
        CR_KERNEL_BASE=`echo $CR_KERNEL | sed -e 's:smp\|enterprise\|bigmem\|hugemem::g'`
	if test ${sysconfdir} = /etc; then
          CR_MODULE_DIR="/lib/modules/${LINUX_VER}/${PACKAGE}"
	else
          CR_MODULE_DIR="${libdir}/${PACKAGE}/${LINUX_VER}"
	fi
       ])
     ])
  CR_IF([test $cr_linux_dir_ok = yes],[$3],[$4])
])

# CR_CHECK_KBUILD()
# ------------------------------------------------------
# Sets up the automake to kbuild interface
AC_DEFUN([CR_CHECK_KBUILD],[
  AC_MSG_CHECKING([for parameters to interface GNU automake with Linux kbuild])
  # Check for legal configuration and handle defaults
  if test x$HAVE_LINUX_2_4 = xyes; then
    if test x$enable_kbuild = xdefault; then
      enable_kbuild=no
    fi
  elif test x$HAVE_LINUX_2_6 = xyes; then
    if test x$enable_kbuild = xno; then
      AC_CACHE_SAVE
      AC_MSG_ERROR([[cannot specify --disable-kbuild with a 2.6.x kernel]])
    else
      enable_kbuild=yes;  # might have been "default"
    fi
  else
    AC_MSG_RESULT([failed])
    AC_MSG_ERROR([Internal error - unknown kernel version])
  fi
  # Configure
  AC_SUBST([KBUILD_CLEAN_FILES])
  AC_SUBST([KBUILD_TARGET_SUFFIX])
  AC_SUBST([KBUILD_MAKE_ARGS])
  if test x$enable_kbuild = xyes; then
    if grep KBUILD_EXTMOD ${LINUX_SRC}/Makefile >/dev/null 2>/dev/null; then
      KBUILD_MAKE_ARGS='KBUILD_EXTMOD=$(builddir)'
      if test ${LINUX_OBJ} != ${LINUX_SRC}; then
	KBUILD_MAKE_ARGS="${KBUILD_MAKE_ARGS} O=${LINUX_OBJ}"
      fi
      if test x$enable_kbuild_verbose = xyes; then
	KBUILD_MAKE_ARGS="${KBUILD_MAKE_ARGS} V=1"
      fi
      AC_MSG_RESULT([ok - set KBUILD_EXTMOD])
    else
      KBUILD_MAKE_ARGS='SUBDIRS=$(builddir) modules'
      AC_MSG_RESULT([ok - set SUBDIRS])
    fi
    if test x$HAVE_LINUX_2_6 = xyes; then
      KBUILD_CLEAN_FILES='*.o *.ko *.mod.c .*.cmd .tmp_versions'
      KBUILD_TARGET_SUFFIX='.ko'
    else
      KBUILD_CLEAN_FILES='*.o .*.o.flags'
      KBUILD_TARGET_SUFFIX='.o'
    fi
    # XXX: should actualy try to compile something
  else
    AC_MSG_RESULT([not used])
  fi
])

# CR_SET_KCFLAGS()
# ------------------------------------------------------
# Sets the architecture dependent CR_K*FLAGS SUBSTs
AC_DEFUN([CR_SET_KCFLAGS],[
  # XXX: Need to use kbuild either here or in CR_TRY_KERNEL_COMPILE
  AC_MSG_CHECKING([[for flags to compile Linux kernel modules]])
  AC_ARG_VAR([KCFLAGS], [prepended to CFLAGS when compiling kernel modules])
  AC_ARG_VAR([KCPPFLAGS], [prepended to CPPFLAGS when compiling kernel modules])
  AC_ARG_VAR([KLDFLAGS], [appended to LDFLAGS when linking kernel modules])
  AC_REQUIRE([AC_CANONICAL_HOST])
  AC_REQUIRE([CR_CHECK_LINUX])
  cr_kcppflags="-I$LINUX_SRC/include -D__KERNEL__ -DMODULE $CR_KTYPE_CPPFLAGS"
  if test "$LINUX_OBJ" != "$LINUX_SRC"; then
    cr_kcppflags="-I$LINUX_OBJ/include -I$LINUX_OBJ/include2 $cr_kcppflags"
  fi
  cr_kcflags="-Wall -Wstrict-prototypes -O2 -fomit-frame-pointer -fno-strict-aliasing -fno-common"
  case $host_cpu in
    i?86)
	cr_kcflags="$cr_kcflags -fno-strength-reduce"
	# XXX: Kludge - this is an example of where kbuild is really needed
	if test -d $LINUX_SRC/include/asm-i386/mach-default; then
	  if test "$LINUX_OBJ" != "$LINUX_SRC"; then
	    cr_kcppflags="$cr_kcppflags -I$LINUX_OBJ/include2/asm/mach-default"
	  fi
	  cr_kcppflags="$cr_kcppflags -I$LINUX_SRC/include/asm-i386/mach-default"
	fi
	;;
    x86_64)
	cr_kcflags="$cr_kcflags -mno-red-zone -mcmodel=kernel -fno-reorder-blocks -finline-limit=2000 -fno-strength-reduce"
	;;
    sparc)
	cr_kcflags="$cr_kcflags -m32 -mno-fpu -fcall-used-g5 -fcall-used-g7"
	;;
    alpha)
	cr_kcflags="$cr_kcflags -mno-fp-regs -ffixed-8 -mcpu=ev5"
	;;
    ppc)
	cr_kcppflags="$cr_kcppflags -D__powerpc__"
	cr_kcflags="$cr_kcflags -fsigned-char -msoft-float -pipe \
		    -ffixed-r2 -Wno-uninitialized -mmultiple -mstring"
	;;
    *)
	AC_MSG_WARN([Unknown host cpu "$host_cpu", using defaults])
	;;
  esac
  # Prepending the following magic excludes /usr/include, but keeps the
  # compiler-specific headers for things like stdarg.h
  cr_kcppflags="-nostdinc -iwithprefix include $cr_kcppflags"
  AC_SUBST([CR_KCFLAGS],[$cr_kcflags])
  AC_SUBST([CR_KCPPFLAGS],[$cr_kcppflags])
  AC_SUBST([CR_KLDFLAGS],['-S -X -r'])
  AC_MSG_RESULT([ok])
])

# CR_DEFINE(VARIABLE,TEST,[TEMPLATE_TEXT])
# ------------------------------------------------------
# Wrapper to invoke AC_DEFINE for a boolean, and optionally AH_TEMPLATE
AC_DEFUN([CR_DEFINE],[
  if [$2]; then
    AC_DEFINE([$1], [1])
  else
    AC_DEFINE([$1], [0])
  fi
  m4_ifval([$3], AH_TEMPLATE([$1], [$3]))
])

# CR_DEFINE_AND_SET(VARIABLE,TEST,[TEMPLATE_TEXT])
# ------------------------------------------------------
# Wrapper like CR_DEFINE which also sets the corresponding shell variable
# to either 1 or the empty string
AC_DEFUN([CR_DEFINE_AND_SET],[
  if [$2]; then
    AC_DEFINE([$1], [1])
    eval "$1=1"
  else
    AC_DEFINE([$1], [0])
    eval "$1=''"
  fi
  m4_ifval([$3], AH_TEMPLATE([$1], [$3]))
])

# CR_ARG_ENABLE(NAME,HELP,[DEFAULT])
# ------------------------------------------------------
# Our own wrapper for AC_ARG_ENABLE
# Default for DEFAULT is "no"
AC_DEFUN([CR_ARG_ENABLE],[
  AC_ARG_ENABLE([$1],
    [AC_HELP_STRING([--enable-$1], [$2])],
    :,
    [[enable_]patsubst([$1], -, _)=m4_default([$3], [no])])
])

# CR_WITH_COMPONENTS(LIST)
# ------------------------------------------------------
# If no flag is passed, then ALL components are built.
# If a flag IS passed, then ONLY the given components are built.
# This is for internal/maintainer use only, thus no --help.
# ex: --with-components=bin,lib
AC_DEFUN([CR_WITH_COMPONENTS],[
  AC_ARG_WITH([components])
  case x"$with_components" in
    xyes)
      AC_MSG_ERROR([--with_components requires an argument.  Known components are: $1]);
      ;;
    x)
      for cr_lcv in $1; do
	eval cr_build_${cr_lcv}=yes
      done
      ;;
    *)
      for cr_lcv in $1; do
	if expr "$with_components" : '.*'"$cr_lcv" >/dev/null; then
	  eval cr_build_${cr_lcv}=yes
	else
	  eval cr_build_${cr_lcv}=no
	fi
      done
      ;;
  esac
])

# CR_CHECK_SIGNUM
# ------------------------------------------------------
# Probes for value to assign to CR_SIGNUM
AC_DEFUN([CR_CHECK_SIGNUM],[
AC_MSG_CHECKING(for value of CR_SIGNUM)
cr_signum="failed"
SAVE_LIBS="$LIBS"
LIBS="-ldl -lpthread $LIBS"
AC_TRY_RUN([#include <stdio.h>
extern int __libc_allocate_rtsig(int);
int main()
{
  FILE *f=fopen("conftestval", "w");
  int s=__libc_allocate_rtsig(0);
  if (!f || s<=0) return(1);
  fprintf(f, "%d\n", s);
  return(0);
}], cr_signum=`cat conftestval`, AC_MSG_ERROR([Failed to probe CR_SIGNUM]), AC_MSG_ERROR([Cannot probe CR_SIGNUM when cross compiling]))
AC_SUBST([CR_SIGNUM],[$cr_signum])
LIBS="$SAVE_LIBS"
AC_MSG_RESULT([$cr_signum])
])

# CR_CONFIG_REPORT
# ------------------------------------------------------
# Reports certain probed/configured values
AC_DEFUN([CR_CONFIG_REPORT],[
  echo "======================================================================"
  echo "Please review the following configuration information:"
  echo "  Kernel source directory = $LINUX_SRC"
  echo "  Kernel build directory = $LINUX_OBJ"
  echo "  Kernel symbol table = ${LINUX_SYSTEM_MAP}${LINUX_VMLINUX}"
  echo "  Kernel version probed from kernel build = $LINUX_VER"
  echo "  Kernel running currently = `uname -r`"
  echo "======================================================================"
])

# CR_TRY_GCC_FLAG(FLAG,[ACTION-IF-TRUE],[ACTION-IF_FALSE])
# ------------------------------------------------------
# Wrapper to invoke AC_TRY_COMPILE for checking a gcc flag
AC_DEFUN([CR_TRY_GCC_FLAG],[
  AC_MSG_CHECKING(whether gcc accepts $1)
  SAVE_CFLAGS=$CFLAGS
  CFLAGS="$CFLAGS $1"
  AC_TRY_COMPILE([], [], [cr_result="yes"],[cr_result="no"])
  CFLAGS=$SAVE_CFLAGS
  AC_MSG_RESULT($cr_result)
  CR_IF([test x"$cr_result" = "xyes"], [$2], [$3])
])

# CR_EGREP_KERNEL_CPP(PATTERN,PROGRAM,[ACTION-IF-TRUE],[ACTION-IF_FALSE])
# ------------------------------------------------------
# Wrapper to invoke AC_EGREP_CPP against the kernel headers
AC_DEFUN([CR_EGREP_KERNEL_CPP],[
  AC_REQUIRE([CR_SET_KCFLAGS])
  SAVE_CPPFLAGS=$CPPFLAGS
  CPPFLAGS="$CR_KCPPFLAGS $CR_KBUILD_EXTRA"
  AC_EGREP_CPP([$1],[
		 #include <linux/kernel.h>
		 #include <linux/types.h>
		 $2],[cr_result="yes"],[cr_result="no"])
  CPPFLAGS=$SAVE_CPPFLAGS
  CR_IF([test x"$cr_result" = "xyes"], [$3], [$4])
])


# CR_EGREP_KERNEL_HEADER(PATTERN,HEADER,[ACTION-IF-TRUE],[ACTION-IF_FALSE])
# ------------------------------------------------------
# Wrapper to invoke (approximately) AC_EGREP_HEADER against the kernel headers
AC_DEFUN([CR_EGREP_KERNEL_HEADER],[
  CR_EGREP_KERNEL_CPP([$1],[#include <$2>],[$3],[$4])
])

# CR_TRY_KERNEL_COMPILE(INCLUDES,PROGRAM,[ACTION-IF-TRUE],[ACTION-IF_FALSE])
# ------------------------------------------------------
# Wrapper to invoke AC_TRY_COMPILE against the kernel headers
AC_DEFUN([CR_TRY_KERNEL_COMPILE],[
  AC_REQUIRE([CR_SET_KCFLAGS])
  SAVE_CFLAGS=$CFLAGS
  SAVE_CPPFLAGS=$CPPFLAGS
  CFLAGS="$CR_KCFLAGS"
  CPPFLAGS="$CR_KCPPFLAGS $CR_KBUILD_EXTRA"
  AC_TRY_COMPILE([
		 #include <linux/kernel.h>
		 #include <linux/types.h>
		 $1],
		 [$2], [cr_result="yes"],[cr_result="no"])
  CFLAGS=$SAVE_CFLAGS
  CPPFLAGS=$SAVE_CPPFLAGS
  CR_IF([test x"$cr_result" = "xyes"], [$3], [$4])
])

# CR_CHECK_KERNEL_COMPILE(SYMBOL,INCLUDES,PROGRAM,[TEMPLATE_TEXT])
# ------------------------------------------------------
# Wrapper to invoke CR_TRY_KERNEL_COMPILE then AC_DEFINE.
# Calls AH_TEMPLATE with either default or supplied text.
# Also sets HAVE_... to 1 or empty string
AC_DEFUN([CR_CHECK_KERNEL_COMPILE],[
  AC_MSG_CHECKING([kernel for $1])
  CR_TRY_KERNEL_COMPILE([$2],[$3],[cr_result=yes],[cr_result=no])
  CR_DEFINE_AND_SET(AS_TR_CPP(HAVE_[]$1),[test x"$cr_result" = xyes],
    m4_default([$4], [[Define to 1 if the kernel has $1]]))
  AC_MSG_RESULT([$cr_result])
])

# CR_CHECK_KERNEL_HEADER(FILE,[INCLUDES],[TEMPLATE_TEXT])
# ------------------------------------------------------
# Wrapper to invoke CR_CHECK_KERNEL_COMPILE for a given header
AC_DEFUN([CR_CHECK_KERNEL_HEADER],[
  CR_CHECK_KERNEL_COMPILE([$1],[
   $2
   #include <$1>
  ],[ ])
])

# CR_CHECK_KERNEL_SYMBOL(SYMBOL,[INCLUDES],[TEMPLATE_TEXT])
# ------------------------------------------------------
# Wrapper to invoke CR_CHECK_KERNEL_COMPILE for a given symbol
AC_DEFUN([CR_CHECK_KERNEL_SYMBOL],[
  CR_CHECK_KERNEL_COMPILE([$1],[$2],[
     int x = sizeof(&$1);
  ])
])

# CR_CHECK_KERNEL_CALL(SYMBOL,[INCLUDES],[TEMPLATE_TEXT])
# ------------------------------------------------------
# Wrapper to invoke CR_CHECK_KERNEL_COMPILE for a given function or macro
AC_DEFUN([CR_CHECK_KERNEL_CALL],[
  CR_CHECK_KERNEL_COMPILE([$1],[$2],[
   #ifdef $1
     /* OK, it exists and is a macro */
   #else
     /* Check for function case */
     int x = sizeof(&$1);
   #endif
  ])
])

# CR_CHECK_KERNEL_PROTOTYPE(SYMBOL,[INCLUDES],[TEMPLATE_TEXT])
# ------------------------------------------------------
# Wrapper to invoke CR_CHECK_KERNEL_COMPILE for a given function prototype
AC_DEFUN([CR_CHECK_KERNEL_PROTOTYPE],[
  CR_CHECK_KERNEL_COMPILE([$1 prototype],[$2],[
     /* Check for function case */
     int x = sizeof(&$1);
  ])
])

# CR_CHECK_KERNEL_TYPE(NAME,[INCLUDES],[TEMPLATE_TEXT])
# ------------------------------------------------------
# Wrapper to invoke CR_CHECK_KERNEL_COMPILE for a given type
AC_DEFUN([CR_CHECK_KERNEL_TYPE],[
  CR_CHECK_KERNEL_COMPILE([$1],[$2],[
     $1 y;
     int x = sizeof(y);
  ])
])

# CR_CHECK_KERNEL_MEMBER(SHORTNAME,INCLUDES,AGGREGATE,TYPE,MEMBER,[TEMPLATE_TEXT])
# ------------------------------------------------------
# Wrapper to invoke CR_CHECK_KERNEL_COMPILE for a given aggregate member w/ type check
AC_DEFUN([CR_CHECK_KERNEL_MEMBER],[
  case "$4" in
    '')		# No type check (e.g. function pointer)
      cr_check=['int y = sizeof(x.$5);'] ;;
    void*\*)	# Pointer to void
      cr_check=['x.$5 = NULL;'] ;;
    *\*)	# Pointer to non-void
      cr_check=['$4 y; *y = *(x.$5);'] ;;
    *)		# Not a pointer
      cr_check=['$4 y = x.$5;'] ;;
  esac
  CR_CHECK_KERNEL_COMPILE([$1],[$2],[$3 x; $cr_check])
])

# CR_CHECK_KERNEL_CONSTANT(SYMBOL,[INCLUDES],[TEMPLATE_TEXT])
# ------------------------------------------------------
# Wrapper to invoke CR_CHECK_KERNEL_COMPILE for a given constant
AC_DEFUN([CR_CHECK_KERNEL_CONSTANT],[
  CR_CHECK_KERNEL_COMPILE([$1],[$2],[
     typeof($1) y;
     int x = sizeof(y);
  ])
])

# CR_CHECK_KERNEL_MACRO(SYMBOL,[INCLUDES],[TEMPLATE_TEXT])
# ------------------------------------------------------
# Wrapper to invoke CR_CHECK_KERNEL_COMPILE for a given macro
AC_DEFUN([CR_CHECK_KERNEL_MACRO],[
  CR_CHECK_KERNEL_COMPILE([$1],[$2],[
   #ifndef $1
     choke me
   #endif
  ])
])

# CR_BAD_KERNEL([REASON])
# ------------------------------------------------------
# Die, informing the user that the kernel cannot be used.
# Optionally provide a reason.
AC_DEFUN([CR_BAD_KERNEL],[
  AC_CACHE_SAVE
  CR_CONFIG_REPORT
  m4_ifval([$1],
   [ AC_MSG_ERROR([Unable to use kernel $LINUX_VER - $1]) ],
   [ AC_MSG_ERROR([Unable to use kernel $LINUX_VER]) ])
])

# CR_LINUX_SYMTAB()
# ------------------------------------------------------
# Check for System.map in some standard places
# If LINUX_SYSTEM_MAP is set, just verifies existance,
# else if LINUX_VMLINUX is set, just verifies existance,
# else search for either file in standard locations.
# When complete sets LINUX_SYMTAB_CMD such that
# "eval $LINUX_SYMTAB_CMD" will produce a System.map on stdout.
cr_stripped_maps=''
AC_DEFUN([_CR_CHECK_SYSTEM_MAP],[
  if test -n "$1" -a -r "$1" && grep ' [[AB]] _end' <"$1" >/dev/null 2>/dev/null; then
    if grep -B1 '[[AB]] _end' <"$1" | grep _stext >/dev/null 2>/dev/null; then
      # Reject "stripped" files (such as in FC2)
      # Recognized (poorly) by _stext and _end as last two entries.
      cr_stripped_maps="$cr_stripped_maps $1"
    else 
      LINUX_SYSTEM_MAP="$1"
      LINUX_SYMTAB_FILE="$1"
      LINUX_SYMTAB_CMD="cat $1 2>/dev/null"
    fi
  fi
])
AC_DEFUN([_CR_CHECK_VMLINUX],[
  AC_REQUIRE([AC_PROG_NM])
  if test -n "$1" -a -r "$1" && ($NM "$1" | grep ' [[AB]] _end') >/dev/null 2>/dev/null; then
    LINUX_VMLINUX="$1"
    LINUX_SYMTAB_FILE="$1"
    LINUX_SYMTAB_CMD="$NM $1 2>/dev/null"
  fi
])
AC_DEFUN([CR_LINUX_SYMTAB],[  
  AC_REQUIRE([CR_CHECK_LINUX])
  AC_MSG_CHECKING([[for Linux kernel symbol table]])
  AC_SUBST([LINUX_SYMTAB_FILE])
  AC_SUBST([LINUX_SYMTAB_CMD])
  AC_SUBST([LINUX_SYMTAB_CONF])
  LINUX_SYMTAB_CMD=""
  # First try validating the user's selection
  if test -n "$LINUX_SYSTEM_MAP" ; then
    _CR_CHECK_SYSTEM_MAP(["$LINUX_SYSTEM_MAP"])
    if test -z "$LINUX_SYMTAB_CMD"; then
      # The user specified a file, but we can't use it.  Abort.
      AC_MSG_RESULT([failed])
      AC_MSG_ERROR([Failed to validate "$LINUX_SYSTEM_MAP"])
    fi
  fi
  if test -z "$LINUX_SYMTAB_CMD" -a -n "$LINUX_VMLINUX" ; then
    _CR_CHECK_VMLINUX(["$LINUX_VMLINUX"])
    if test -z "$LINUX_SYMTAB_CMD"; then
      # The user specified a file, but we can't use it.  Abort.
      AC_MSG_RESULT([failed])
      AC_MSG_ERROR([Failed to validate "$LINUX_VMLINUX"])
    fi
  fi
  # Next try searching for System.map or vmlinux in standard locations
  # Note we use the kernel version found in the headers, not `uname -r`
  if test -z "$LINUX_SYMTAB_CMD" ; then
    for cr_file_pattern in "$LINUX_OBJ/@-$LINUX_VER" \
			   "/boot/@-$LINUX_VER"      \
			   "/@-$LINUX_VER"           \
			   "$LINUX_OBJ/@"            \
			   "/usr/lib/debug/boot/@-$LINUX_VER" \
			   "/usr/lib/debug/lib/modules/$LINUX_VER/@" \
			; do
      # Try System.map in the given location
      cr_file=`echo $cr_file_pattern | sed -e 's|@|System.map|'`
      _CR_CHECK_SYSTEM_MAP([$cr_file])
      if test -n "$LINUX_SYMTAB_CMD"; then
	break
      fi
      # Try vmlinux in the given location
      cr_file=`echo $cr_file_pattern | sed -e 's|@|vmlinux|'`
      _CR_CHECK_VMLINUX([$cr_file])
      if test -n "$LINUX_SYMTAB_CMD"; then
	break
      fi
    done
  fi
  # Announce our result
  if test -z "$LINUX_SYMTAB_CMD"; then
    LINUX_SYMTAB_CMD="true"
    AC_MSG_RESULT([failed])
    if test -n "$cr_stripped_maps"; then
      AC_MSG_WARN([Skipped stripped System.map file(s): $cr_stripped_maps])
      AC_MSG_ERROR([Failed to locate kernel symbol table.  Try installing the kernel-debuginfo package matching your kernel, or using --with-system-map or --with-vmlinux.])
    else
      AC_MSG_ERROR([Failed to locate kernel symbol table.  Try using --with-system-map or --with-vmlinux.])
    fi
  else
    AC_MSG_RESULT([$LINUX_SYMTAB_FILE])
  fi
  # Now check for consistency w/ the kernel source
  # XXX: Currently just check SMPness. Can this be more aggressive?
  AC_MSG_CHECKING([for SMP kernel source])
  CR_TRY_KERNEL_COMPILE([
	#include <linux/autoconf.h>
	#ifndef CONFIG_SMP
	    choke me
	#endif
	],[],[cr_kernel_smp=yes],[cr_kernel_smp=no])
  AC_MSG_RESULT([$cr_kernel_smp]);
  AC_MSG_CHECKING([for SMP kernel symbol table])
  cr_symtab_smp=no
  if test -n "`eval $LINUX_SYMTAB_CMD | grep del_timer_sync 2>/dev/null`"; then
    cr_symtab_smp=yes
  fi
  AC_MSG_RESULT([$cr_symtab_smp]);
  if test "$cr_kernel_smp" != "$cr_symtab_smp"; then
    CR_CONFIG_REPORT
    if test "$cr_kernel_smp" = yes; then
      AC_MSG_ERROR([Kernel source is configured SMP but the kernel symbol table is not.  Consider specifying a symbol table with --with-system-map or --with-vmlinux.  Or, if using kernel sources that are configured by /boot/kernel.h, you may try --with-kernel-type=UP to force a uni-processor interpretation of the sources.])
    else
      AC_MSG_ERROR([Kernel source is configured uni-processor but the kernel symbol table is SMP.  Consider specifying a symbol table with --with-system-map or --with-vmlinux.  Or, if using kernel sources that are configured by /boot/kernel.h, you may try --with-kernel-type=SMP to force an SMP interpretation of the sources.])
    fi
  fi
])

AC_DEFUN([CR_KSYM_PATTS],[
  case "$CR_ARCH" in
    ppc64)
      CR_KSYM_PATTERN_DATA=['[bBdDrR] ']
      CR_KSYM_PATTERN_CODE=['[dD] ']   dnl Function descriptor is data
      ;;
    *)
      CR_KSYM_PATTERN_DATA=['[bBdDrR] ']
      CR_KSYM_PATTERN_CODE=['[tT] ']
      ;;
  esac
])
		      

# CR_FIND_KSYM(SYMBOL,TYPE)
# ------------------------------------------------------
# Search System.map for address of the given symbol and
# do one of three things in $(CONFIG_HEADER):
# If not found, leave CR_K${TYPE}_foo undefined
# If found to be exported, "#define CR_K${TYPE}_foo 0"
# If found not to be exported, "#define CR_K${TYPE}_foo 0x<value>"
AC_DEFUN([CR_FIND_KSYM],[
  AC_REQUIRE([CR_LINUX_SYMTAB])
  AC_REQUIRE([CR_KSYM_PATTS])
  AC_MSG_CHECKING([[kernel symbol table for $1]])
  cr_pattern=`eval echo \"\\${CR_KSYM_PATTERN_$2}[$1]$\"`
  cr_addr=`eval $LINUX_SYMTAB_CMD | sed -n -e "/$cr_pattern/ {s/ .*//p;}"`
  if test -z $cr_addr; then
    cr_result="not found"
  else
    if eval $LINUX_SYMTAB_CMD | grep " __ksymtab_$1\$" >/dev/null; then
      cr_result="exported"
      cr_addr="0"
    else
      cr_result="$cr_addr"
      cr_addr="0x$cr_addr"
      CR_[$2]_IMPORTS="$CR_[$2]_IMPORTS $1"
    fi
    AC_DEFINE_UNQUOTED(CR_K[$2]_[$1],$cr_addr,
		[Define to address of non-exported kernel symbol $1, or 0 if exported])
  fi
  AC_MSG_RESULT([$cr_result])
])


# CR_FIND_EXPORTED_KSYM(SYMBOL,TYPE)
# ------------------------------------------------------
# Search System.map for address of the given symbol and
# add the following to $(CONFIG_HEADER):
#   "#define CR_EXPORTED_K${TYPE}_foo 0x<value>"
# If not found, cr_addr wil be 0 on return.
AC_DEFUN([CR_FIND_EXPORTED_KSYM],[
  AC_REQUIRE([CR_LINUX_SYMTAB])
  AC_REQUIRE([CR_KSYM_PATTS])
  AC_MSG_CHECKING([[kernel symbol table for exported $1]])
  cr_pattern=`eval echo \"\\${CR_KSYM_PATTERN_$2}[$1]$\"`
  cr_addr=`eval $LINUX_SYMTAB_CMD | sed -n -e "/$cr_pattern/ {s/ .*//p;}"`
  if test -n "$cr_addr" && eval $LINUX_SYMTAB_CMD | grep " __ksymtab_$1\$" >/dev/null; then
    cr_result="$cr_addr"
    cr_addr="0x$cr_addr"
    AC_DEFINE_UNQUOTED(CR_EXPORTED_K[$2]_[$1],$cr_addr,
		[Define to address of exported kernel symbol $1])
  else
    cr_result="not found"
    cr_addr="0"
  fi
  AC_MSG_RESULT([$cr_result])
])

# Find how to rpmbuild (if possible at all)
AC_DEFUN([CR_PROG_RPMBUILD],[
  AC_PATH_PROGS([RPMBUILD],[rpmbuild rpmb rpm],[none],[$PATH:/usr/lib/rpm])
  if $RPMBUILD -bs 2>&1 | grep 'no spec' >/dev/null 2>/dev/null; then
    :
  else
    RPMBUILD=none
  fi
])

# CR_CONFIG_SSS()
# ------------------------------------------------------
# Configure Scalable Systems Software stuff
AC_DEFUN([CR_CONFIG_SSS],[
  AC_MSG_CHECKING([for MPD_HOME])
  AC_ARG_WITH([mpd],
    [AC_HELP_STRING([--with-mpd=DIR],[specify path to the mpd installation directory (default is to search using PATH)])],
    [AS_IF([test $with_mpd != yes],[MPD_HOME=$with_mpd])])
  if test -n "$MPD_HOME"; then
    AC_MSG_RESULT([$MPD_HOME])
  else
    AC_MSG_RESULT([not set])
    AC_PATH_PROG([MPDCMD],[mpd],[not found],[${PATH}:/usr/mpi/bin])
    if test "$MPDCMD" = "not found"; then
      AC_MSG_ERROR([--enable-sss requires path to the mpd installation directory which could not be found in the default location, or by using PATH.  Please use --with-mpd=dir to specify the mpd installation directory.])
    else
      MPD_HOME=`dirname $MPDCMD`
      MPD_HOME=`dirname $MPD_HOME`
    fi
  fi
  SAVE_CPPFLAGS=$CPPFLAGS
  SAVE_LDFLAGS=$LDFLAGS
  CPPFLAGS="$CPPFLAGS -I${MPD_HOME}/include"
  LDFLAGS="$LDFLAGS -L${MPD_HOME}/lib"
  # First just check for availability
  AC_CHECK_LIB([mpich],[PMI_Init],[:],[AC_MSG_ERROR([Unable to link to PMI library required by --enable-sss])])
  AC_CHECK_HEADERS([pmi.h])
  # Next check for sufficiently recent versions
  AC_CHECK_LIB([mpich],[PMI_Get_id],[:],[AC_MSG_ERROR([${MPD_HOME}/lib/libmpich.a contains an older unsupported PMI version.])])
  if test "$ac_cv_header_pmi_h" = "yes"; then
    AC_MSG_CHECKING([for pmi.h sufficiently recent])
    AC_TRY_COMPILE([#include <pmi.h>],[
		    int err, is_init;
		    err = PMI_Initialized(&is_init);
		   ], [cr_result="yes"],[cr_result="no"])
    AC_MSG_RESULT($cr_result)
    if test "$cr_result" != "yes"; then
      AC_MSG_ERROR([${MPD_HOME}/pmi.h is from an older unsupported PMI version.])
    fi
  fi
  CPPFLAGS=$SAVE_CPPFLAGS
  LDFLAGS=$SAVE_LDFLAGS
  AC_SUBST(MPD_HOME)
  AC_DEFINE_UNQUOTED([MPD_HOME], "$MPD_HOME")
  AH_TEMPLATE([MPD_HOME],[MPD installation directory (if needed)])

  AC_MSG_CHECKING([for LAM_HOME])
  AC_ARG_WITH([lam],
    [AC_HELP_STRING([--with-lam=DIR],[specify path to LAM/MPI installation directory (default is to search using PATH)])],
    [AS_IF([test $with_lam != yes],[LAM_HOME=$with_lam])])
  # Note that we must not try to verify LAM_HOME, since in a normal build
  # sequence we must be installed before the corresponding LAM can be built.
  if test -n "$LAM_HOME"; then
    AC_MSG_RESULT([$LAM_HOME])
  else
    AC_MSG_RESULT([not set])
    AC_PATH_PROG([LAMCMD],[lamboot],[not found])
    if test "$LAMCMD" = "not found"; then
      AC_MSG_ERROR([--enable-sss requires the path to the LAM/MPI installation directory which could not be found using PATH.  Please use --with-lam=dir to specify the lam/mpi installation directory.])
    else
      LAM_HOME=`dirname $LAMCMD`
      LAM_HOME=`dirname $LAM_HOME`
    fi
  fi
  AC_SUBST(LAM_HOME)
  AC_DEFINE_UNQUOTED([LAM_HOME], "$LAM_HOME")
  AH_TEMPLATE([LAM_HOME],[LAM/MPI installation directory (if needed)])
])

# CR_STACK_DIRECTION()
# ------------------------------------------------------
# Determine direction of stack grown
# Copied (and slightly modified) from autoconf's alloca code
AC_DEFUN([CR_STACK_DIRECTION],[
  AC_TRY_RUN([
    int find_stack_direction (void) {
      static char *addr = 0;
      auto char dummy;
      if (addr == 0) {
        addr = &dummy;
        return find_stack_direction();
      } else {
        return (&dummy > addr) ? 0 : 1;
      }
    }
    int main(void) {
      return find_stack_direction();
    }], [cr_stack_direction=1],
	[cr_stack_direction=-1])
  AC_DEFINE_UNQUOTED([CR_STACK_GROWTH], $cr_stack_direction)
  AH_TEMPLATE([CR_STACK_GROWTH],[Positive if stack grows up, negative if it grows down])
])


