/*
 * Berkeley Lab Checkpoint/Restart (BLCR) for Linux is Copyright (c)
 * 2003, The Regents of the University of California, through Lawrence
 * Berkeley National Laboratory (subject to receipt of any required
 * approvals from the U.S. Dept. of Energy).  All rights reserved.
 *
 * Portions may be copyrighted by others, as may be noted in specific
 * copyright notices within specific files.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: imports.c,v 1.15 2007/02/08 20:39:22 phargrov Exp $
 */

#ifndef EXPORT_SYMTAB
  #define EXPORT_SYMTAB
#endif

#include "blcr_config.h"

#include <linux/autoconf.h>
#if defined(CONFIG_SMP) && ! defined(__SMP__)
  #define __SMP__
#endif
#if defined(CONFIG_MODVERSIONS) && ! defined(MODVERSIONS)
  #define MODVERSIONS
#endif
#include <linux/module.h>
#include "blcr_ksyms.h"

/* Prototypes and declarations for things we export here */
#include "blcr_imports.h"


/* This comment contains "EXPORT_SYMBOL" to trigger proper kbuild magic */

#if defined(CR_KDATA_child_reaper) && CR_KDATA_child_reaper
  CR_IMPORT_KDATA(child_reaper)
#endif
#if defined(CR_KDATA_sys_call_table) && CR_KDATA_sys_call_table
  CR_IMPORT_KDATA(sys_call_table)
#endif
#if defined(CR_KDATA_cpu_gdt_table) && CR_KDATA_cpu_gdt_table
  #include <asm/desc.h>
  CR_IMPORT_KDATA(cpu_gdt_table)            // i386 and x86_64
#endif
#if defined(CR_KDATA_per_cpu__cpu_gdt_descr)
  #if defined(CR_KDATA_per_cpu__cpu_gdt_descr) && CR_KDATA_per_cpu__cpu_gdt_descr
    #include <asm/desc.h>
    CR_IMPORT_KDATA(per_cpu__cpu_gdt_descr)
  #else
    /* Exported */
  #endif
#elif HAVE_CPU_GDT_DESCR && defined(CR_KDATA_cpu_gdt_descr) && CR_KDATA_cpu_gdt_descr
  #include <asm/desc.h>
  CR_IMPORT_KDATA(cpu_gdt_descr)
#endif
#if defined(CR_KDATA_mmu_cr4_features) && CR_KDATA_mmu_cr4_features
  #include <asm/processor.h>
  CR_IMPORT_KDATA(mmu_cr4_features)            // i386 and x86_64
#endif
#if defined(CR_KDATA_write_pipe_fops) && CR_KDATA_write_pipe_fops
  CR_IMPORT_KDATA(write_pipe_fops)
#endif
#if defined(CR_KDATA_rdwr_pipe_fops) && CR_KDATA_rdwr_pipe_fops
  CR_IMPORT_KDATA(rdwr_pipe_fops)
#endif
#if defined(CR_KDATA_read_pipe_fops) && CR_KDATA_read_pipe_fops
  CR_IMPORT_KDATA(read_pipe_fops)
#endif
#if defined(CR_KDATA_write_fifo_fops) && CR_KDATA_write_fifo_fops
  CR_IMPORT_KDATA(write_fifo_fops)
#endif
#if defined(CR_KDATA_rdwr_fifo_fops) && CR_KDATA_rdwr_fifo_fops
  CR_IMPORT_KDATA(rdwr_fifo_fops)
#endif
#if defined(CR_KDATA_read_fifo_fops) && CR_KDATA_read_fifo_fops
  CR_IMPORT_KDATA(read_fifo_fops)
#endif
#if defined(CR_KDATA_pidmap_array) && CR_KDATA_pidmap_array
  CR_IMPORT_KDATA(pidmap_array)
#endif
#if defined(CR_KDATA_pidmap_lock) && CR_KDATA_pidmap_lock
  CR_IMPORT_KDATA(pidmap_lock)
#endif
#if HAVE_ALLOC_PID
  #if defined(CR_KDATA_pid_hash) && CR_KDATA_pid_hash
    CR_IMPORT_KDATA(pid_hash)
    CR_IMPORT_KDATA(pidhash_shift)
  #endif
  #if defined(CR_KDATA_pid_cachep) && CR_KDATA_pid_cachep
    CR_IMPORT_KDATA(pid_cachep)
  #endif
  #if defined(CR_KDATA_init_pspace) && CR_KDATA_init_pspace
    CR_IMPORT_KDATA(init_pspace)
  #endif
#endif
#if defined(CR_KDATA_anon_pipe_buf_ops) && CR_KDATA_anon_pipe_buf_ops
  CR_IMPORT_KDATA(anon_pipe_buf_ops)
#endif
#if defined(CR_KDATA_tasklist_lock) && CR_KDATA_tasklist_lock
  CR_IMPORT_KDATA(tasklist_lock)
#endif


#if HAVE_ALLOC_PID
  #if defined(CR_KCODE_put_pid) && CR_KCODE_put_pid
    CR_IMPORT_KCODE(put_pid)
  #endif
#endif
#if defined(CR_KCODE_sys_dup2 ) && CR_KCODE_sys_dup2 
  CR_IMPORT_KCODE(sys_dup2)
#endif
#if defined(CR_KCODE_sys_lseek ) && CR_KCODE_sys_lseek 
  CR_IMPORT_KCODE(sys_lseek)
#endif
#if defined(CR_KCODE_sys_ftruncate ) && CR_KCODE_sys_ftruncate 
  CR_IMPORT_KCODE(sys_ftruncate)
#endif
#if defined(CR_KCODE_sys_close) && CR_KCODE_sys_close
  CR_IMPORT_KCODE(sys_close)
#endif
#if defined(CR_KCODE_sys_ptrace) && CR_KCODE_sys_ptrace
  CR_IMPORT_KCODE(sys_ptrace)
#endif
#if defined(CR_KCODE_sys_setresuid) && CR_KCODE_sys_setresuid
  CR_IMPORT_KCODE(sys_setresuid)
#endif
#if defined(CR_KCODE_sys_setresgid) && CR_KCODE_sys_setresgid
  CR_IMPORT_KCODE(sys_setresgid)
#endif
#if defined(CR_KCODE_sys_setgroups) && CR_KCODE_sys_setgroups
  CR_IMPORT_KCODE(sys_setgroups)
#endif
#if defined(CR_KCODE_do_pipe) && CR_KCODE_do_pipe
  CR_IMPORT_KCODE(do_pipe)
#endif
#if defined(CR_KCODE_detach_pid) && CR_KCODE_detach_pid
  CR_IMPORT_KCODE(detach_pid)
#endif
#if defined(CR_KCODE_attach_pid) && CR_KCODE_attach_pid
  CR_IMPORT_KCODE(attach_pid)
#endif
#if defined(CR_KCODE_find_pid) && CR_KCODE_find_pid
  CR_IMPORT_KCODE(find_pid)
#endif
#if defined(CR_KCODE_link_pid) && CR_KCODE_link_pid
  CR_IMPORT_KCODE(link_pid)
#endif
#if defined(CR_KCODE___put_task_struct) && CR_KCODE___put_task_struct
  CR_IMPORT_KCODE(__put_task_struct)
#endif
#if defined(CR_KCODE___put_task_struct_cb) && CR_KCODE___put_task_struct_cb
  CR_IMPORT_KCODE(__put_task_struct_cb)
#endif
#if defined(CR_KCODE_path_lookup) && CR_KCODE_path_lookup
  CR_IMPORT_KCODE(path_lookup) // missing in early 2.4 kernels
#endif
#if defined(CR_KCODE_vfs_read) && CR_KCODE_vfs_read
  CR_IMPORT_KCODE(vfs_read)
#endif
#if defined(CR_KCODE_vfs_write) && CR_KCODE_vfs_write
  CR_IMPORT_KCODE(vfs_write)
#endif
#if defined(CR_KCODE_rw_verify_area) && CR_KCODE_rw_verify_area
  CR_IMPORT_KCODE(rw_verify_area)
#endif
#if defined(CR_KCODE_copy_siginfo_to_user) && CR_KCODE_copy_siginfo_to_user
  CR_IMPORT_KCODE(copy_siginfo_to_user)
#endif
#if defined(CR_KCODE_set_fs_pwd) && CR_KCODE_set_fs_pwd
  CR_IMPORT_KCODE(set_fs_pwd)
#endif
#if defined(CR_KCODE_put_filp) && CR_KCODE_put_filp
  CR_IMPORT_KCODE(put_filp)
#endif
#if defined(CR_KCODE_sys_mprotect) && CR_KCODE_sys_mprotect
  CR_IMPORT_KCODE(sys_mprotect)
#endif
#if defined(CR_KCODE_sys_mremap) && CR_KCODE_sys_mremap
  CR_IMPORT_KCODE(sys_mremap)
#endif
#if defined(CR_KCODE_do_exit) && CR_KCODE_do_exit
  CR_IMPORT_KCODE(do_exit)
#endif
#if defined(CR_KCODE_do_sigaction) && CR_KCODE_do_sigaction
  CR_IMPORT_KCODE(do_sigaction)
#endif
#if defined(CR_KCODE_do_execve) && CR_KCODE_do_execve
  CR_IMPORT_KCODE(do_execve)
#endif
#if defined(CR_KCODE_load_gs_index) && CR_KCODE_load_gs_index
  CR_IMPORT_KCODE(load_gs_index)  // x86_64
#endif
#if defined(CR_KCODE_expand_files) && CR_KCODE_expand_files
  CR_IMPORT_KCODE(expand_files)
#endif
#if defined(CR_KCODE_arch_setup_additional_pages) && CR_KCODE_arch_setup_additional_pages
  CR_IMPORT_KCODE(arch_setup_additional_pages)
#endif


MODULE_AUTHOR("Lawrence Berkeley National Lab " PACKAGE_BUGREPORT);
MODULE_DESCRIPTION("Berkeley Lab Checkpoint/Restart symbol imports kernel module");
#ifdef MODULE_LICENSE
MODULE_LICENSE("GPL");
#endif

