/*
 * Berkeley Lab Checkpoint/Restart (BLCR) for Linux is Copyright (c)
 * 2003, The Regents of the University of California, through Lawrence
 * Berkeley National Laboratory (subject to receipt of any required
 * approvals from the U.S. Dept. of Energy).  All rights reserved.
 *
 * Portions may be copyrighted by others, as may be noted in specific
 * copyright notices within specific files.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: cr_barrier.c,v 1.9 2007/01/19 22:02:23 phargrov Exp $
 */

#include "cr_module.h"

void cr_barrier_init(cr_barrier_t *barrier, int count)
{
    init_waitqueue_head(&barrier->wait);
    atomic_set(&barrier->count, count);
    barrier->once = 0;
}

// Called to pass a barrier, blocking for all others to do so too.
// See cr_barrier.h for more information.
int cr_barrier_enter(cr_barrier_t *barrier)
{
    if (atomic_dec_and_test(&barrier->count)) {
	wake_up(&barrier->wait);
    } else {
	wait_event(barrier->wait, (atomic_read(&barrier->count) == 0));
    }
    return !test_and_set_bit(0, &barrier->once);
}

// Called to pass a barrier, possibly waking any waiters
// Can be called by a third party if somebody dies unexpectedly.
void cr_barrier_notify(cr_barrier_t *barrier)
{
    if (atomic_dec_and_test(&barrier->count)) {
	wake_up(&barrier->wait);
    }
}

// Called to wait on a barrier
int cr_barrier_wait(cr_barrier_t *barrier)
{
    wait_event(barrier->wait, (atomic_read(&barrier->count) == 0));
    return !test_and_set_bit(0, &barrier->once);
}
