/*
 * Berkeley Lab Checkpoint/Restart (BLCR) for Linux is Copyright (c)
 * 2003, The Regents of the University of California, through Lawrence
 * Berkeley National Laboratory (subject to receipt of any required
 * approvals from the U.S. Dept. of Energy).  All rights reserved.
 *
 * Portions may be copyrighted by others, as may be noted in specific
 * copyright notices within specific files.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: cr_barrier.h,v 1.8 2007/01/19 22:02:23 phargrov Exp $
 */

#ifndef _CR_BARRIER_H
#define _CR_BARRIER_H        1

#include <linux/module.h>
#include <linux/spinlock.h>
#include <linux/wait.h>

// Single-use barrier which can be "notified" by a "third party" (such
// as when a task dies).
//
// A call to cr_barrier_{wait,enter}() will not return until the barrier
// has been passed "count" times.  The term "passing the barrier"
// is taken to mean either a call to cr_barrier_enter() or to
// cr_barrier_notify().
//
// Exactly one call to cr_barrier_{wait,enter}() will return non-zero.

typedef struct cr_barrier_s {
	wait_queue_head_t	wait;
	atomic_t		count;	// How many remain
	unsigned long		once;
} cr_barrier_t;

extern void cr_barrier_init(cr_barrier_t *barrier, int count);
extern int cr_barrier_enter(cr_barrier_t *barrier);

/* split-phase treatment */
extern void cr_barrier_notify(cr_barrier_t *barrier);
extern int cr_barrier_wait(cr_barrier_t *barrier);

#endif
