/* 
 * Berkeley Lab Checkpoint/Restart (BLCR) for Linux is Copyright (c)
 * 2003, The Regents of the University of California, through Lawrence
 * Berkeley National Laboratory (subject to receipt of any required
 * approvals from the U.S. Dept. of Energy).  All rights reserved.
 *
 * Portions may be copyrighted by others, as may be noted in specific
 * copyright notices within specific files.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: blcr_imports.h,v 1.6 2007/02/08 20:39:26 phargrov Exp $
 *
 * This file provides otherwise missing declarations for functions
 * and data which BLCR imports via the blcr_imports kernel module.
 */
#ifndef _CR_PROTO_H
#define _CR_PROTO_H	1

#include "blcr_config.h"

#include <linux/autoconf.h>
#if defined(CONFIG_SMP) && ! defined(__SMP__)
  #define __SMP__
#endif
#if defined(CONFIG_MODVERSIONS) && ! defined(MODVERSIONS)
  #define MODVERSIONS
#endif
#include <linux/module.h>

#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/version.h>
#include <linux/list.h>
#include <linux/wait.h>
#include <linux/sched.h>
#include <linux/fs.h>
#include <linux/file.h>
#include <linux/mount.h>
#include <linux/slab.h>
#include <linux/highmem.h>
#include <linux/spinlock.h>
#include <asm/atomic.h>
#include <linux/unistd.h>
#include <asm/unistd.h>
#if HAVE_LINUX_NAMEI_H
  #include <linux/namei.h>
#endif
#if HAVE_LINUX_SYSCALLS_H
  #include <linux/syscalls.h>           /* for sys_close, etc. */
#endif
#if HAVE_LINUX_SYSCALL_H
  #include <linux/syscall.h>           /* for sys_close, etc. */
#endif
#if HAVE_LINUX_PSPACE_H
  #include <linux/pspace.h>           /* for struct pidmap */
#endif
#if HAVE_ASM_ELF_H
  #include <asm/elf.h>
#endif


/* Unconditional declarations.
 *
 * Either all kernels lack these, or we know we can redeclare w/o conflict.
 * Some of these may only exist conditionally, or be unused under some
 * situations, but are still safe to declare.
 */
extern void * sys_call_table[];
extern struct pipe_buf_operations anon_pipe_buf_ops;
extern spinlock_t pidmap_lock;

extern asmlinkage long sys_setresuid(uid_t, uid_t, uid_t);
extern asmlinkage long sys_setresgid(gid_t, gid_t, gid_t);
extern asmlinkage long sys_setgroups(int, gid_t *);


/* Conditional declarations.
 *
 * Some variation in types (e.g. architecture-specific sizes, presence or
 * absence of "const") prevents us from declaring these unconditionally, or
 * the declaration is invalid on some kernels.
 */
#if defined(CR_KDATA_child_reaper) && CR_KDATA_child_reaper
    extern struct task_struct *child_reaper;
#endif
#if !HAVE_SYS_DUP2_PROTOTYPE
    extern asmlinkage long sys_dup2(unsigned int oldfd, unsigned int newfd);
#endif
#if !HAVE_SYS_LSEEK_PROTOTYPE
    extern asmlinkage off_t sys_lseek(unsigned int fd, off_t offset, unsigned int origin);
#endif
#if !HAVE_SYS_FTRUNCATE_PROTOTYPE
    extern asmlinkage long sys_ftruncate(unsigned int fd, unsigned long length);
#endif
#if !HAVE_SYS_PTRACE_PROTOTYPE
    extern asmlinkage int sys_ptrace(unsigned long request, unsigned long pid, unsigned long addr, unsigned long data);
#endif
#if !HAVE_SYS_MPROTECT_PROTOTYPE
    extern asmlinkage long sys_mprotect(unsigned long start, size_t len, unsigned long prot);
#endif
#if !HAVE_SYS_MREMAP_PROTOTYPE
    extern asmlinkage unsigned long sys_mremap(unsigned long addr, unsigned long old_len, unsigned long new_len, unsigned long flags, unsigned long new_addr);
#endif
#if !HAVE_EXPAND_FILES_PROTOTYPE
    extern int expand_files(struct files_struct *, int);
#endif
#if !HAVE_STRUCT_PIDMAP
    struct pidmap {
	atomic_t nr_free;
	void *page;
    };
#endif
#if defined(CR_KDATA_pidmap_array) && CR_KDATA_pidmap_array
    extern struct pidmap pidmap_array[];
#endif
#if HAVE_ALLOC_PID
    #if defined(CR_KDATA_pid_hash) && CR_KDATA_pid_hash
        extern struct hlist_head *pid_hash;
        extern int pidhash_shift;
    #endif
    #if defined(CR_KDATA_pid_cachep) && CR_KDATA_pid_cachep
        extern kmem_cache_t *pid_cachep;
    #endif
#endif
#if HAVE_CONST_FOPS /* typeof() is "cooler", but breaks module crc generation */
    #define CR_FOPS_CONST const
#else
    #define CR_FOPS_CONST /* empty */
#endif
    extern CR_FOPS_CONST struct file_operations write_pipe_fops;
    extern CR_FOPS_CONST struct file_operations rdwr_pipe_fops;
    extern CR_FOPS_CONST struct file_operations read_pipe_fops;
    extern CR_FOPS_CONST struct file_operations write_fifo_fops;
    extern CR_FOPS_CONST struct file_operations rdwr_fifo_fops;
    extern CR_FOPS_CONST struct file_operations read_fifo_fops;
#undef CR_FOPS_CONST


#endif
