#!/usr/bin/env python

from commands import getstatusoutput
import re
from os import environ, getuid, stat, mkdir, makedirs, access, W_OK
from os.path import basename, isfile
from stat import *
from pwd import getpwuid
from sss.node import Node
from sss.server import SSSSelectServer, AddEvent, StateLoadError, StateSaveError
from sss.ssslib import comm_lib, InitError, ConnectError, CommunicationError
from sys import argv,exc_info,exit, stderr
from syslog import openlog,syslog,closelog, LOG_INFO,LOG_ERR, LOG_LOCAL0
from time import strftime, localtime

from elementtree.ElementTree import Element, SubElement, XML, tostring

from xml.dom.minidom import Document,parseString

# DUMB: hard-coded paths to mpich and checkpoint/restart 
mpichbin = "/usr/mpi/bin"
bindir = '/usr/bin'    # directory where BLCR executables are installed

def verbose(msg):
    if verbose_on:
        print msg

class PGID(Node):
    __attrfields__ = ['pgid']

    def __setdefaults__(self):
        self.pgid = 'bogus_pgid'

class CID(Node):
    __attrfields__ = ['cid']

    def __setdefaults__(self):
        self.cid = 'bogus_cid'

class process:
    def __init__(self, rank, host, pid, user):
        self.rank = rank
        self.host = host
        self.pid = pid
        self.user = user

class restartSpec:
    def __init__(self, rank, host, user, contextfile):
        self.rank = rank
        self.host = host
        self.user = user
        self.contextfile = contextfile

class ProcessNode(Node):
    __attrfields__=['host','pid','session']

class ProcessSpecNode(Node):
    typemap={'process':ProcessNode,}

class ProcessGroupNode(Node):
    __attrfields__=['pgid','submitter','output','totalprocs','state']
    typemap={'process-spec':ProcessSpecNode}

class ProcessGroupsNode(Node):
    typemap={'process-groups':ProcessGroupNode}

# one context.pid file for each process in process group
# - prefix name with process manager process group ID
def getContextFileName(pgid, pid):
    return '%s/%s_context.%s' % (context_dir, pgid, pid)

# one info file for each process group
def getInfoFileName(pgid):
    return '%s/%s_info' % (context_dir, pgid)

def getprocessinfoSSS(pgid):
    # create request to get node/pid info for processes in checkpoint
    msg = Document().createElement('get-process-group-info')
    pspec =  Document().createElement('process')
    pspec.setAttribute("host", '*')
    pspec.setAttribute("pid", '*')
    pspec.setAttribute("exec", '*')

    pg =  Document().createElement('process-group')
    pg.setAttribute("pgid", '%s' % pgid)
    pg.setAttribute("submitter", '*')
    pg.setAttribute("totalprocs", '*')
    pg.setAttribute("state", '*')

    pg.appendChild(pspec)
    msg.appendChild(pg)

    print "---OLD request for process info\n"
    print msg.toprettyxml()

    msg = Element('get-process-group-info')
    pg = Element('process-group',  pgid=pgid)
    SubElement(pg, 'process', host='*', pid='*')
    msg.append(pg)


    #SubElement(pg, 'host-spec')

    #pspec = Element('process', host='*', pid='*')
    # 'exec' is a Python keyword, so can't be passed as named argument
    #pspec.set('exec', '*')
    #msg.append(pspec)
    #SubElement(pspec, 'process-group', pgid='%s'%pgid, submitter='*',
    #                                   totalprocs='*', state='*')

    print "---NEW request for process info\n%s" % tostring(msg)

    process_manager = comm.ClientInit('process-manager')
    comm.SendMessage(process_manager,tostring(msg))
    ack = comm.RecvMessage(process_manager)
    comm.ClientClose(process_manager)
    pgs = ProcessGroupsNode(parseString(ack).documentElement)

    print "\n-----ack----\n%s" % ack

    procs = []
    rank = 0
    for child in filter(lambda x: x.tagname == 'process', 
                        pgs.children[0].children):
        procs.append(process(rank, child.host, child.pid))
        rank += 1
    return procs

# hacked version that makes sure SIGCHLD isn't ignored
def mygetstatusoutput(cmd):
    """Return (status, output) of executing cmd in a shell."""
    import os, signal
    # need to use default handler, else pipe.close barfs
    oldhandler = signal.signal(signal.SIGCLD, signal.SIG_DFL)
    pipe = os.popen('{ ' + cmd + '; } 2>&1', 'r')
    text = pipe.read()
    sts = pipe.close()
    if sts is None: sts = 0
    if text[-1:] == '\n': text = text[:-1]
    signal.signal(signal.SIGCLD, oldhandler)
    return sts, text

# HACK: call mpdlistjobs until process manager fixed
def getprocessinfoMPD(pgid):
    procs = []
    (err, liststr) = mygetstatusoutput('%s/mpdlistjobs' % mpichbin)
    #(err, liststr) = getstatusoutput('%s/mpdlistjobs' % mpichbin)
    #print "err=%s, liststr=%s" % (err, liststr)
    if err:
        raise "unable to run %s/mpdlistjobs" % mpichbin
    chopped = re.split('\n\n', liststr)
    for listing in chopped:
        match = re.search(r'^jobalias\s*=\s*(\S+)$', listing, re.MULTILINE)
        if not match:
            raise "Error parsing jobalias from mpdlistjobs"
        jobid = match.group(1)
        if jobid == pgid:
            match = re.search(r'^rank\s*=\s*(\d+)$', listing, re.MULTILINE)
            if not match:
                raise "Error parsing rank from mpdlistjobs"
            rank = match.group(1)
            match = re.search(r'^host\s*=\s*(\S+)$', listing, re.MULTILINE)
            if not match:
                raise "Error parsing host from mpdlistjobs"
            host = match.group(1)
            match = re.search(r'^pid\s*=\s*(\d+)$', listing, re.MULTILINE)
            if not match:
                raise "Error parsing pid from mpdlistjobs"
            pid = match.group(1)
            match = re.search(r'^username\s*=\s*(\S+)$', listing, re.MULTILINE)
            if not match:
                raise "Error parsing username from mpdlistjobs"
            user = match.group(1)
            procs.append(process(rank, host, pid, user))
    return procs

def makeCheckpointRequest(procs, signal, pgid):
    output = 'merged'
    msg = Element('create-process-group', pgid='*',
                  totalprocs=`len(procs)`, 
                  submitter=submitter,
                  output=output)
    procs.sort(lambda a,b: cmp(a.rank, b.rank)) # list in rank order, 
                                                # to correspond w/hosts
    hosts = ''; i = 0
    for proc in procs:
        # hosts must be globbed into single string in rank order
        if i: hosts += '\n'
        hosts += proc.host 
        i = i + 1
        # 1 process-spec per cr_checkpoint, since args differ
        ps = Element('process-spec', range='%s' % proc.rank, user=proc.user,
                     path=environ.get('PATH'), cwd=cwd )
        ps.set('exec', checkpoint_exe)
        msg.append(ps)
        idx = 1
        if signal:
            for arg in ('--signal', `signal`):
                SubElement(ps, 'arg', idx=`idx`, value=arg)
                idx += 1
        for arg in ('-f', getContextFileName(pgid, proc.pid), proc.pid):
            SubElement(ps, 'arg', idx=`idx`, value=arg)
            idx += 1
    # add host-spec: separate element from process-spec, due to poor design
    hs = Element('host-spec')
    hs.text = hosts
    msg.append(hs)
    return msg

def getRestartInfo(pgid):
    infofile = getInfoFileName(pgid)
    procs = []
    f = open(infofile, 'r')
    for line in f.readlines():
        m = re.match(r'(\d+):([^:]+):([^:]+):([^:\n]+)', line)
        if not m:
            raise "Invalid line in restart info file '%s':\n  " %\
                    (infofile, line)
        procs.append(restartSpec(m.group(1), m.group(2), m.group(3), 
                                 m.group(4)))
    return procs

def makeRestartRequest(cid, signal):
    output = 'merged'
    procs = getRestartInfo(cid)
    msg = Element('create-process-group', pgid='*',
                  totalprocs=`len(procs)`, 
                  submitter=submitter,
                  output=output)
    hosts = '';  i = 0
    for proc in procs:
        # hosts must be globbed into single string in rank order
        if i: hosts += '\n'
        hosts += proc.host 
        i = i + 1
        # 1 process-spec per cr_checkpoint, since args differ
        ps = Element('process-spec', range='%s' % proc.rank, user=proc.user,
                     path=environ.get('PATH'), cwd=cwd )
        ps.set('exec', restart_exe)
        msg.append(ps)
        idx = 1
        if signal:
            for arg in ('--signal', `signal`):
                SubElement(ps, 'arg', idx=`idx`, value=arg)
                idx += 1
        for arg in (proc.contextfile, ):
            SubElement(ps, 'arg', idx=`idx`, value=arg)
            idx += 1
    # add host-spec: separate element from process-spec, due to poor design
    hs = Element('host-spec')
    hs.text = hosts
    msg.append(hs)
    return msg

class CheckpointManager(SSSSelectServer):
    __implementation__='crmg'
    __component__='checkpoint-manager'

    def __parseinput__(self):
        self.validate=0
        self.dispatch={"checkpoint-request":self.HandleXMLCheckpointRequest,
                       "suspend-request":self.HandleXMLSuspendRequest,
                       "resume-request":self.HandleXMLResumeRequest}
        if debug:
            print "I'm waiting..."

    def PostEvent(self,m,d):
        e = Element("add-event")
        t = strftime("%c",localtime())
        SubElement(e, "event", component="checkpoint-manager", msg=m, data=d, time=t)
        print "event-to-send=%s" % tostring(e)
        try:
            h=comm.ClientInit("event-manager")
            comm.SendMessage(h,tostring(e))
            comm.RecvMessage(h)
            comm.ClientClose(h)
        except:
            if debug:
                print "Failed to send event"
        del e

    def HandleXMLCheckpointRequest(self,xml,(peer,port)):
        if debug:
            print "Got checkpoint request"
        reply = Document().createElement("checkpoint-result")        
        return self.HandleCkRequest(xml, (peer,port), reply, 0)

    def HandleXMLSuspendRequest(self,xml,(peer,port)):
        if debug:
            print "Got suspend request"
        reply = Document().createElement("suspend-result")        
        return self.HandleCkRequest(xml, (peer,port), reply, 15)

    def HandleCkRequest(self,xml,(peer,port), reply, signal):
        pgidObj = PGID(xml)
        pgid = pgidObj.pgid

        print "request=%s" % xml.toprettyxml()

        self.PostEvent("checkpoint-request", "pgid=%s,signal=%s" % (pgid,signal))

        #procs = getprocessinfoSSS(pgid)
        procs = getprocessinfoMPD(pgid)
        if debug:
            for p in procs:
                print "proc: rank=%s, host=%s, pid=%s" % \
                        (p.rank, p.host, p.pid)
        if not len(procs):
            syslog(LOG_ERR, "%s: Process group %s not found" % (moi, pgid))
            if debug:
                print "%s: Process group %s not found: ignoring" % (moi, pgid)
            return reply

        req = makeCheckpointRequest(procs, signal, pgid)
        if debug:
            print "----Checkpoint request----\n%s" % tostring(req)
        
        #comm = comm_lib(debug='-d' in sys.argv)
        process_manager = comm.ClientInit('process-manager')
        comm.SendMessage(process_manager, tostring(req))
        ack = comm.RecvMessage(process_manager)
        comm.ClientClose(process_manager)
        ack_dom = parseString(ack)
        cr_pgid=ack_dom.documentElement.getAttribute('pgid')
        if debug:
            print "Got PGID %s for checkpoint pgid" % cr_pgid

        # save info needed to restart
        rf = open(getInfoFileName(pgid), 'w')
        for proc in procs:
            cfile = getContextFileName(pgid, proc.pid)
            rf.write("%s:%s:%s:%s\n" % (proc.rank, proc.host, proc.user, cfile)) 
        rf.close()
                                   
        reply.setAttribute("cid", pgid)
        #if debug:
        #    print "----- reply ------"
        #    print reply.toprettyxml()
        return reply

    def HandleXMLResumeRequest(self, xml, (peer,port)):
        if debug:
            print "Got resume request"
            print xml.toprettyxml()

        cidObj = CID(xml)
        cid = cidObj.cid
        if debug:
            print "Got CID=%s" % cid

        self.PostEvent("restart-request", "cid=%s" % cid)

        reply = Document().createElement("resume-result")        

        req = makeRestartRequest(cid, 0)
        if debug:
            print "Restart request"
            print tostring(req)
        
        process_manager = comm.ClientInit('process-manager')
        #comm.SendMessage(process_manager, req.toxml())
        comm.SendMessage(process_manager, tostring(req))
        ack = comm.RecvMessage(process_manager)
        comm.ClientClose(process_manager)
        ack_dom = parseString(ack)
        rs_pgid=ack_dom.documentElement.getAttribute('pgid')
        if debug:
            print "Got PGID %s for restart pgid\nwaiting for event data" \
                    % (rs_pgid)
        reply.setAttribute("pgid", rs_pgid)
        return reply

def makeContextSubdir(subdir):
    try:
        sinfo = stat(subdir) 
        if not S_ISDIR(sinfo[ST_MODE]):
            raise "'%s' exists but is not a directory!" % subdir
    except OSError, osx:
        makedirs(subdir)


if __name__ == "__main__":
    from getopt import getopt, GetoptError
    moi = basename(argv[0])
    daemon = 0
    debug = 0
    verbose_on = 0
    context_dir = ''
    try:
        (opts, args)=getopt(argv[1:],'dv',['daemon=', 'context-dir=', 'bindir='])
    except GetoptError, msg:
        print "%s\nUsage:\ncrmg.py [-d] --context-dir=/path/for/checkpoints [--daemon <pidfile>]"%(msg)
        exit(1)
    for (opt, arg) in opts:
        if opt == '-d': debug = 1
        elif opt == '-v': verbose_on = 1
        elif opt == '--daemon': daemon = arg
        elif opt == '--context-dir':
            context_dir = arg
            if not re.match(r'/', context_dir):
                raise "context-dir '%s' is not a full path" % context_dir
        elif opt == '--bindir':
           bindir = arg 

    if not context_dir:
        raise "must provide --context-dir=/directory/for/storing/checkpoints"

    global checkpoint_exe, restart_exe 
    checkpoint_exe = '%s/cr_checkpoint' % bindir
    restart_exe = '%s/cr_restart' % bindir
    if not isfile(checkpoint_exe): 
        stderr.write("Error: '%s' does not exist, or is not a regular file.\n" % checkpoint_exe)
        stderr.write("Use '--bindir' to point to directory containing cr_checkpoint\n")
        exit(-1)
    if not isfile(restart_exe): 
        stderr.write("Error: '%s' does not exist, or is not a regular file.\n" % restart_exe)
        stderr.write("Use '--bindir' to point to directory containing cr_restart\n")
        exit(-1)

    comm = comm_lib(debug=debug)
    submitter = getpwuid(getuid())[0]
    cwd = environ['PWD']

    try:
        try:
            sinfo = stat(context_dir) 
            if not S_ISDIR(sinfo[ST_MODE]):
                raise "Context directory '%s' does not exist!" % context_dir
            if not access(context_dir, W_OK):
                raise "--context-dir '%s' is not writable" % context_dir
        except OSError, osx:
            mkdir(context_dir)
        s = CheckpointManager(debug=debug)
    except InitError, msg:
        print "Library init Error:%s"%(msg)
        exit(1)

    if daemon:
        from sss.daemonize import daemonize
        daemonize(daemon)

    s.ServeForever()

